import os
import sys
import json
import re

#normalize files(reference and automatic transcript) for running edit distance. Version 1.0
#Steps
#1. Eliminate punctuations(commas, poornvirams, ....,?,[],(),-,etc) and replace them with spaces
#2. Eliminate english unicodes(eg. unclear,etc) and replace them with spaces
#3. Put each hindi word on a separate line
#4. Run edit distance on each time stamp and keep track of the distance from one time stamp to the other
#5. The edit distance for the whole file is then:edit(file)= edit(t1) + ... + edit(tn) and WER = edit(file)/Total words(in manual transcript)
#6. For edit distance algorithm, treat each word as a symbol from some alphabet
#7. For edit distance algorithm, for testing two words are equal, develop some formal scheme of comparing their similarity in meanings.

def remove_all(alist,elem):
	while(elem in alist):
		alist.remove(elem)

def do_stuff(fname):
	fp = open(fname,'r')
	f = open('normalized_'+fname,'w')
	i = 0
	#Generate a new file as per steps 1,2,3.
	for line in fp:
		if(line!='\n'):	
			if(i == 0 or line.find('Chatra')!=-1):
				i=i+1
				f.write(line)
				continue	
			l = line	
			r = re.findall(r"[0-9]+\.[0-9]+\s{0,}-\s{0,}[0-9]+\.[0-9]+\s{0,}\w+\s{0,}-?\s{0,}",line)
			if(len(r)!=0):
				index = line.find(r[0]) + len(r[0]) -1
				l = line[index:]
				f.write('t'+'\n')
	
			
			for word in l.split():
				#remove danda
				char_list = list(word)
				remove_all(char_list,'\u0964')
				word = ''.join(char_list)
				word_list = re.split(',+|\.+|\[+|\]+|\(+|\)+|\?+',word)
				for w in word_list:
					if(w!=''):
						f.write(w+'\n')
			#print(line)
		
		i = i+1
	f.write('t'+'\n')	
	fp.close()
	f.close()
#Done steps 1,3



import sys
import os
import json
import shutil

class Time(object):
    mins = 0
    sec = 0
    def __init__(self,mins,sec):
		self.mins = mins
		self.sec = sec
    def changeMins(self,mins):
    	self.mins = mins
    def changeSecs(self,sec):
    	self.sec = sec
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

def integrate_transcripts(audio_file_name):
	'''
	Given the audio file name, this function will make one file(MainTranscript.txt) by concatenating all the chunks of this audio file
	'''
	out_path = os.path.join(os.getcwd(), 'Out_Chunks')
	#now combine them into a single file
	transcripts = os.listdir(out_path)
	#first create ordered transcript on the basis of chunk numbers
	#print(transcripts)
	for i in range(0, len(transcripts)):
		number = int(transcripts[i][9:-4])
		transcripts[i] = number

	transcripts.sort()
	#print(transcripts)
	F = open('MainTranscript.txt','w')
	F.write(audio_file_name)
	F.write('\n\n\n')
	cur_time = Time(0,0)
	ite = 0
	for transcript in transcripts:
		name = 'out_chunk' + str(transcript) + '.txt'
		f = open(os.path.join(out_path, name), 'r')
		if(cur_time.sec >= 60):
			cur_time.changeSecs(cur_time.sec - 60)
			cur_time.changeMins(cur_time.mins + 1)
		string1 = "{0:0>2}".format(cur_time.mins)
		string2 = "{0:0>2}".format(cur_time.sec)
		if(ite<2):
			cur_time.changeSecs(cur_time.sec+14)
		else:
			cur_time.changeSecs(cur_time.sec+13)
		if(cur_time.sec >= 60):
			cur_time.changeSecs(cur_time.sec - 60)
			cur_time.changeMins(cur_time.mins + 1)
		string3 = "{0:0>2}".format(cur_time.mins)
		string4 = "{0:0>2}".format(cur_time.sec)
		data = json.load(f)
		if(len(data)!=0):
			F.write(string1 + '.' + string2 + ' - ' + string3 + '.' + string4 + ' sec - ')
			if(isinstance(data, str)):
				F.write(data[u"results"][0][u"alternatives"][0][u"transcript"])
			else:
				value = data["results"][0]["alternatives"][0]["transcript"]
				value = value.encode('utf-8')
				F.write(value)
			F.write('\n\n\n')
			cur_time.changeSecs(cur_time.sec + 1)
		ite = ite + 1
	F.close()
	shutil.rmtree(out_path)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------



'''
This is the main library file which will have all the basic functions, we want to provide to the user
We will keep on appending gender, transcripts, silence removal and quality classification parts to this file

Authors: Jayanth Reddy, Mohammad Wasih, Aaditeshwar Seth
Last Modified: 7th August, 2018
'''

'''
Necessory Utilities:

HIGH LEVEL:
1. Remove silence from audio 
2. Get Gender of audio
3. Obtain Transcripts of audio
4. Obtain quality of audio

LOW LEVEL:
> get_feature_representation of an audio
> train neural_network for gender
> train svm for quality
> get accuracy of transcript
...
'''
import os
import sys
import pkg_resources
import numpy as np
from sklearn.preprocessing import StandardScaler
from keras.models import load_model
from subprocess import Popen
#done with standard imports

import Gender_Classification.integrate as integ
import Gender_Classification.disintegrate as disinteg
import Gender_Classification.mp3_to_wav as mtw
import Gender_Classification.learn as learn
import Gender_Classification.extract_features as exf
import Gender_Classification.excl_http_req as exhr
#done with Gender_Classification imports

import Silence_Removal.sln as sln
#done with Silence_Removal imports

import Automatic_Transcripts.combine_script as cs
import Automatic_Transcripts.script as sc
#done with Automatic_Transcripts imports
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


#LOW LEVEL routines

def train(train_file, test_file):
	'''
	This function will train the neural network;
	train_file is input.csv and test_file is test.csv
	'''
	learn.train_model(train_file, test_file)
#***************************************************---------------------------------------------------------------------------*************************************************	

def authenticate_user(path_to_key):
	'''
	Given path to key.json file(used for authentication), this function will generate access token file
	'''
	os.environ['key_path'] = path_to_key
	p1 = Popen(['gcloud auth activate-service-account --key-file=${key_path}'], shell = 'True')
	p1.wait()
	os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = path_to_key
	p2 = Popen(['gcloud auth application-default print-access-token > access-token'], shell = 'True')
#***************************************************---------------------------------------------------------------------------*************************************************	
	
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------	


#HIGH LEVEL routines

def remove_silence(input_audio):
	'''
	This will produce an audio with silent parts removed
	Input: audio file(in .wav format)
	Output: audio file with silence removed(in .wav format); name is input_audio_rmsilence.wav by default
	'''
	output_audio, ext = os.path.splitext(input_audio)
	output_audio += '_rmsilence.wav'
	sln.remove_silence(input_audio, output_audio)
#***************************************************--------------------------------------------------------------------*********************************************************

def get_gender(input_url, input_audio = None):
	'''
	This will return the predicted gender of the audio file as characterized by its input_url
	Basic Process:
	1. Get mp3 file from input_url
	2. Convert to wav format
	3. Remove silence periods in the wav file
	4. Get feature representation of the wav file by chunking it and appending all features to a list and assume gender is male
	5. Load Gender Classification neural network
	6. Forward pass the feature vector and get the corresponding label
	7. Return the gender(label) to user; male is 0 and female is 1
	'''
	if(input_audio == None):
		exhr.get_audio_from_url(input_url, 'test_point.mp3')
		mtw.convert_mp3_to_wav('test_point.mp3')
		remove_silence('test_point.wav')
		features = np.array(exf.get_features('test_point_rmsilence.wav'))
		print(features)
		model_path = pkg_resources.resource_filename('Indian_Speech_Lib', 'models/gender_nn')
		model = load_model(model_path)
		scaler = StandardScaler()
		scaler.fit(features[:,0:136])
		[a, b] = model.evaluate(scaler.transform(features[:,0:136]), features[:,136])
		if b > 0.5:
			return 0
		else:
			return 1
	
	else:
		mtw.convert_mp3_to_wav(input_audio)
		input_audio_name, ext = os.path.splitext(input_audio)
		input_audio_wav = input_audio_name + '.wav'
		remove_silence(input_audio_wav)
		features = np.array(exf.get_features(input_audio_name + '_rmsilence.wav'))
		model_path = pkg_resources.resource_filename('Indian_Speech_Lib', 'models/gender_nn')
		model = load_model(model_path)
		scaler = StandardScaler()
		scaler.fit(features[:,0:136])
		[a, b] = model.evaluate(scaler.transform(features[:,0:136]), features[:,136])
		if b > 0.5:
			return 0
		else:
			return 1
#***************************************************---------------------------------------------------------------------------*************************************************	

def get_automatic_transcript(input_url, path_to_key, bucket_name, input_audio = None, silence_removal = None):
	'''
	Given an input_url or input_audio in .mp3 format and whether to remove silence or not;
	this function will generate a file, 'AutomaticTrans.txt' which will contain the automatic transcript for this audio
	'''
	
	authenticate_user(path_to_key)
	if(input_audio == None):
		exhr.get_audio_from_url(input_url, 'test_point.mp3')
		mtw.convert_mp3_to_wav('test_point.mp3')
		if(silence_removal):
			remove_silence('test_point.wav')
			sc.get_transcript('test_point_rmsilence.wav', bucket_name)
			cs.integrate_transcripts('test_point_rmsilence.wav')
		else:
			sc.get_transcript('test_point.wav')
			cs.integrate_transcripts('test_point.wav')
	
	else:
		mtw.convert_mp3_to_wav(input_audio)
		input_audio_name, ext = os.path.splitext(input_audio)
		input_audio_wav = input_audio_name + '.wav'
		if(silence_removal):
			remove_silence(input_audio_wav)
			sc.get_transcript(input_audio_name + '_rmsilence.wav')
			cs.integrate_transcripts(input_audio_name + '_rmsilence.wav')
		else:
			sc.get_transcript(input_audio_name)
			cs.integrate_transcripts(input_audio_name)
	
	F = open('AutomaticTrans.txt', 'w')
	f = open('MainTranscript.txt', 'r')
	lines = f.readlines()
	for line in lines:
		F.write(line)
	os.remove('MainTranscript.txt')
	F.close()
#********************************************************---------------------------------------------------------------------**************************************************


























import numpy as np
import csv
import soundfile as sf
from pyAudioAnalysis import audioBasicIO
from pyAudioAnalysis import audioFeatureExtraction
from statistics import mean, stdev
import csv

from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
from keras.models import Sequential
from keras.layers import Dense, Dropout


def train_model(train_file, test_file):
	'''
	train_file is csv for training, test_file is unseen csv data
	'''
	rows = []
	with open(train_file, 'r') as csvfile:
		csvreader = csv.reader(csvfile)
		fields = csvreader.next()
		for row in csvreader:
			rows.append(row)
			
	rows1=[]
	with open(test_file, 'r') as csvfile:
		csvreader = csv.reader(csvfile)
		fields = csvreader.next()
		for row in csvreader:
			rows1.append(row)

	rows = np.array(rows)
	rows1 = np.array(rows1)
	X_train, X_val, y_train, y_val = train_test_split(rows[:,0:136],rows[:,136], random_state=0, test_size=0.2)
	scaler = StandardScaler()
	scaler.fit(X_train)
	X_train = scaler.transform(X_train)
	X_val = scaler.transform(X_val)
	
	X_test = scaler.transform(rows1[:,0:136])
	y_test = rows1[:,136]

	model = Sequential()
	model.add(Dense(25, input_dim=136, activation='relu'))
	model.add(Dropout(0.4))
	model.add(Dense(10, activation='relu'))
	model.add(Dropout(0.4))
	model.add(Dense(1, activation='sigmoid'))

	model.compile(loss='binary_crossentropy',
				  optimizer='rmsprop',
				  metrics=['accuracy'])

	model.fit(X_train, y_train,
			  epochs=10,
			  batch_size=128)
			  
	score = model.evaluate(X_val, y_val, batch_size=128)	#accuracy on validation set
	score1 = model.evaluate(X_test, y_test, batch_size=128)	#accuracy on test(unseen set)
	print(score)
	print(score1)
	#save model now
	model.save('gender_nn')
#***************************************************************------------------------------------------------------------****************************************************

'''
data, samplerate = sf.read('tstm3.wav')
i=0
j=1
temp = samplerate*10
print(temp)
n = (len(data)/temp)
print(n)
while i<len(data):
	sf.write('f'+str(j)+'.wav',data[i:i+temp],samplerate)
	i += temp
	j += 1
l1=[]
for i in range(1,n+1,1):
	l = []
	[Fs, x] = audioBasicIO.readAudioFile("f"+str(i)+".wav")
	#x=audioBasicIO.stereo2mono(x)
	F = audioFeatureExtraction.stFeatureExtraction(x, Fs, 0.05*Fs, 0.025*Fs)
	#print(len(F[0]))
	for j in range(34):
		l.append(min(F[j]))
		l.append(max(F[j]))
		l.append(mean(F[j]))
		l.append(stdev(F[j]))
	l.append(0)
	l1.append(l)
l1 = np.array(l1)
#print(l1[0])
[a,b] = model.evaluate(scaler.transform(l1[:,0:136]),l1[:,136])
if b>0.5:
	print('M')
else:
	print('F')=
'''



import soundfile as sf

def break_into_chunks(input_file, gender):
	'''
	Given an input_file, this function will break this .wav file into chunks of 10s duration each
	gender is the gender of this input file, i.e. 'm' or 'f'
	'''
	data, samplerate = sf.read(input_file)
	i=0
	j=1
	temp = samplerate*10
	n = (len(data)/temp)
	while i<len(data):
		sf.write(gender + str(j) + '.wav', data[i: i+temp], samplerate)
		i += temp
		j += 1
#*******************************************************--------------------------------------------------------------------*****************************************************



import os
from subprocess import Popen

def convert_mp3_to_wav(filename):
	'''
	Given an mp3 file, this function will create a corresponding .wav file in the same location where .mp3 file is there
	'''
	if (filename.endswith(".mp3")): #or .avi, .mpeg, whatever.
		[a, b] = filename.split('.')
		p = Popen(['ffmpeg -i ' + filename + ' -acodec pcm_s16le -ac 1 -ar 16000 ' + a + '.wav'.format(filename)], shell = True)
		p.wait()



from pyAudioAnalysis import audioBasicIO
from pyAudioAnalysis import audioFeatureExtraction
from statistics import mean, stdev
import csv
import timeit
import soundfile as sf


def get_features(input_file, gender = 0):
	'''
	Given an input .wav file and its gender, this function will return a list of lists corresponding to features of each of its chunks
	male is 0
	female is 1;
	by default we will assume gender to be male(0) and then we will compare our test predictions with gender = male and decide what did the model predict the gender as
	'''
	data, samplerate = sf.read(input_file)
	i=0
	j=1
	temp = samplerate*10
	n = (len(data)/temp)
	while i<len(data):
		sf.write('m' + str(j) + '.wav', data[i:i+temp], samplerate)
		i += temp
		j += 1
	l1=[]
	for i in range(1, n+1, 1):
		l = []
		[Fs, x] = audioBasicIO.readAudioFile("m" + str(i) + ".wav")
		F = audioFeatureExtraction.stFeatureExtraction(x, Fs, 0.05*Fs, 0.025*Fs)
		for j in range(34):
			l.append(min(F[j]))
			l.append(max(F[j]))
			l.append(mean(F[j]))
			l.append(stdev(F[j]))
		l.append(gender)
		l1.append(l)
	return l1
#******************************************************-----------------------------------------------------------------------*************************************************

def generate_data(output_csv):
	'''
	This function will read in the entire liste of audio files and extract features from them and append to output_csv
	'''
	l1=[]
	for i in range(1,488,1):
		l = []
		[Fs, x] = audioBasicIO.readAudioFile("f" + str(i) + ".wav")
		F = audioFeatureExtraction.stFeatureExtraction(x, Fs, 0.05*Fs, 0.025*Fs)
		for j in range(34):
			l.append(min(F[j]))
			l.append(max(F[j]))
			l.append(mean(F[j]))
			l.append(stdev(F[j]))
		l.append(1)
		l1.append(l)
	for i in range(1,488,1):
		l = []
		[Fs, x] = audioBasicIO.readAudioFile("m" + str(i) + ".wav")
		F = audioFeatureExtraction.stFeatureExtraction(x, Fs, 0.05*Fs, 0.025*Fs)
		for j in range(34):
			l.append(min(F[j]))
			l.append(max(F[j]))
			l.append(mean(F[j]))
			l.append(stdev(F[j]))
		l.append(0)
		l1.append(l)
	with open(output_csv, "w") as f:
		writer = csv.writer(f)
		writer.writerows(l1)
#******************************************************-----------------------------------------------------------------------**************************************************



import urllib2
import xlrd

def get_audio_from_url(input_url, output_file):
	'''
	Given an audio url, this function will fetch it and generate an mp3 file for it;
	output_file should be a proper name for an mp3, i.e. ending with .mp3
	'''
	mp3file = urllib2.urlopen(input_url)
	FP = open(output_file, 'wb')
	FP.write(mp3file.read())
	FP.close()
#*****************************------------------------------------*******************************	

def get_audio_from_excel(input_file = 'up_hindi_belt_Item_data.xlsx'):
	ExcelFileName = input_file 
	workbook = xlrd.open_workbook(ExcelFileName)
	worksheet = workbook.sheet_by_name("kab_item_data")
	num_rows = worksheet.nrows #Number of Rows
	num_cols = worksheet.ncols #Number of Columns
	m = 0
	f = 0
	for curr_row in range(0,num_rows, 1):
		data = worksheet.cell_value(curr_row, 6)
		if data == 'M':
			m += 1
			s1 = worksheet.cell_value(curr_row, 2)
			mp3file = urllib2.urlopen(s1)
			with open('m' + str(m) + '.mp3', 'wb') as output:
				output.write(mp3file.read())
				
		if data == 'F':
			f += 1
			s2 = worksheet.cell_value(curr_row, 2)
			mp3 = urllib2.urlopen(s2)
			with open('f' + str(f) + '.mp3', 'wb') as outputs:
				outputs.write(mp3.read())
#****************************-------------------------------------********************************



import soundfile as sf
import os
from subprocess import Popen, PIPE
import numpy
import shutil
import json
import sys

#accept a list of audio files and generate a mega transcript file
#input_audio = sys.argv[1:]

#first activate service account and verify credentials and then generate access-token file
os.system('gcloud auth activate-service-account  --key-file=key.json')
os.environ['GOOGLE_APPLICATION_CREDENTIALS']='/home/wasih/Desktop/Speech_Recognition/key.json'
os.system('gcloud auth application-default print-access-token > access-token')

F = open('AutomaticTrans.txt','w')
path = '/home/wasih/Desktop/Speech_Recognition/New/audio'
input_audio = os.listdir(path)

for audio in input_audio:
	print(audio)
	os.environ["audio"]=audio
	p = Popen(['python script.py ${audio}'], shell='true')
	p.wait()
	p = Popen(['python combine_script.py ${audio}'], shell='true')
	p.wait()
	f = open('MainTranscript.txt','r')
	lines = f.readlines()
	for line in lines:
		F.write(line)
	os.remove('MainTranscript.txt')
	print('Audio:'+audio+' done!'+'\n')
	
F.close()
print('Completed!!')




import accuracyv2 as acc
import normalization as norm
import os
import sys

if __name__=="__main__":
	#first obtain normalization of both manual and automatic transcripts
	man_name = sys.argv[1]
	auto_name = sys.argv[2]
	norm.do_stuff(man_name)	#do normalization of manual transcript
	norm.do_stuff(auto_name)	#do normalization of automatic transcript	
	man_normalized = 'normalized_' + man_name
	auto_normalized = 'normalized_' + auto_name
	WER, accuracy = acc.evaluate(man_normalized, auto_normalized)	#evaluate automatic transcript
	print('Avg Word Error Rate = ', WER)
	print('Accuracy = ', accuracy)
	
	
	



from distutils.core import setup
from setuptools import find_packages

setup(
    # Application name:
    name = "Indian_Speech_Lib",

    # Version number (initial):
    version = "1.0.3",

    # Application author details:
    author= "Mohd Wasih, Jayanth Reddy",
    author_email = "wasih.mohd@gmail.com",

    # Packages
    packages = find_packages(),

    # Include additional files into the package
    include_package_data = True,

    # Details
    url="http://pypi.python.org/pypi/Indian_Speech_Lib_v103/",

    #
    # license="LICENSE.txt",
    description = "Useful speech recognition and transcription related library for Indian languages.",

    # long_description=open("README.txt").read(),

    # Dependent packages (distributions)
    install_requires=[
		"tensorflow",
		"pydub",
		"numpy",
		"sklearn",
		"keras",
		"soundfile",
		"pyAudioAnalysis",
		"statistics",
    ],
)



import soundfile as sf
import os
from subprocess import Popen, PIPE
import numpy
import shutil
import json
import sys

def request_write(filename):	
	'''
	This function will write the sync-request.json file for requesting service from the api
	make the json file sync-request.json using dict data structure and filename as uri parameter
	'''
	#dict has key(string):value pairs, separated by commas and enclosed in braces; value can itself be a dict or a list of dicts, and so on.
	d = {"config":{"encoding":"FLAC","languageCode":"hi-IN","enableWordTimeOffsets":False},"audio":{"uri":"gs://hindi786/"+filename}}
	fp = open('sync-request.json','w')
	json.dump(d,fp,indent=4)
	fp.close()
#********************************************************---------------------------------------------------------------*********************************************************

def get_transcript(path_audio_file, bucket_name = 'hindi786'):
	'''
	Given the path/input audio file and the bucket_name for the storage created on cloud,
	this function will obtain the transcripts in an Out_chunks folder
	'''
	os.environ["input"] = path_audio_file
	p1 = Popen(['ffmpeg -i ${input} temp.flac -loglevel quiet'], shell='true')
	p1.wait()
	#convert input .mp3 to temp.flac
	
	p2 = Popen(['ffmpeg -i temp.flac -ac 1 mono.flac -loglevel quiet'], shell='true')
	p2.wait()
	#convert channel to mono
	
	os.remove('temp.flac')
	print('Step 1 Done!')
	#converted audio mp3 to mono.flac.
	
	#now create chunks of 14 size flacs.
	data, rate = sf.read('mono.flac')
	print(rate)
	#do block-wise reading of the file, mono.flac
	path = os.path.join(os.getcwd(), 'Chunks')
	if not os.path.exists(path):
		os.makedirs(path)
	d,r = sf.read('mono.flac', rate * 14, 0)
	sf.write(os.path.join(path, 'chunk0.flac'), d, rate)
	d,r = sf.read('mono.flac', frames = rate * 15, start = rate * 14)
	sf.write(os.path.join(path, 'chunk1.flac'), d, rate)
	
	j = 2
	for block in sf.blocks('mono.flac', blocksize = rate * 14, start = rate * 30):
		sf.write(os.path.join(path, 'chunk') + str(j) + '.flac', block, rate)
		j = j + 1
	
	os.remove('mono.flac')
	print('Step 2 Done!')
	#generated blocks of 14s duration flac files

	#now upload the files on cloud bucket using gsutil
	chunks = os.listdir(path)
	for chunk in chunks:
		os.environ["object"] = chunk
		os.environ["path"] = os.path.join(path, chunk)
		p = Popen(['gsutil cp ${path} gs://' + bucket_name], shell='true')
		p.wait()
		p = Popen(['gsutil acl ch -u AllUsers:R gs://' + bucket_name + '/' + '${object}'], shell='true')
		p.wait()

	shutil.rmtree(path)
	print('Step 3 Done!')
	#uploaded chunks on cloud bucket

	#now call speech api on each of the chunks
	fp = open('access-token', 'r')
	token = fp.readline()
	token = token[:-1]
	os.environ["token"] = token

	out_path = os.path.join(os.getcwd(), 'Out_Chunks')
	for chunk in chunks:
		request_write(chunk)
		out_file = os.path.join(out_path, 'out_' + chunk[:-5] + '.txt')
		p = Popen(['curl -s -H "Content-Type: application/json" \-H "Authorization: Bearer ${token}" \https://speech.googleapis.com/v1/speech:recognize \-d @sync-request.json'], stdout=PIPE, stderr=PIPE, shell = 'True')
		stdout, stderr = p.communicate()
		print stdout, '\n\n'
		if not os.path.exists(out_path):
			os.makedirs(out_path)
		fp = open(out_file, 'w')
		if stdout:
			fp.write(stdout)
		p.wait()

	for chunk in chunks:
		os.environ["file"] = chunk
		p = Popen(['gsutil rm gs://' + bucket_name + '/' + '${file}'], shell = 'True')
		p.wait()

	print('Step 4 Done!')
	#got transcripts for all the chunks
#********************************************************---------------------------------------------------------------*********************************************************



import json
import os
import sys
import re
import itertools

def evaluate(man_file,auto_file):
	f1 = open(man_file,'r')	#manual transcription(normalized)
	f2 = open(auto_file,'r')	#automatic transcription(normalized)

	#4. Run edit distance on each time stamp and keep track of the distance from one time stamp to the other
	#5. The edit distance for the whole file is then:edit(file)= edit(t1) + ... + edit(tn) and WER = edit(file)/Total words(in manual transcript)
	#6. For edit distance algorithm, treat each word as a symbol from some alphabet
	#7. For edit distance algorithm, for testing two words are equal, develop some formal scheme of comparing their similarity in meanings.

	#8. Do this step instead of steps 4,5,6,7: Just forget about timestamps and treat the whole file as a single transcript, i.e. do global alignment.

	man = f1.readlines()
	auto = f2.readlines()
	f1.close()
	f2.close()
	#if(man[0]!=auto[0]):
	#	print('Panic! Manual and Automatic transcripts are for different files')
	#	exit(0)

	manual = []
	automatic = []
	for word in man[1:]:
		if(re.match(r"unclear",word)):
			continue
		elif(re.match(r"blank",word)):
			continue
		else:
			manual.append(word[:-1])

	for word in auto[1:]:
		if(re.match(r"unclear",word)):
			continue
		elif(re.match(r"blank",word)):
			continue
		else:
			automatic.append(word[:-1])

	#manual is the now the string to be aligned with automatic and so apply DP
	def editD(S,T):
		n = len(S)
		m = len(T)
		#need a DP table of size n*m
		#insertions/deletions/modifications are allowed
		DP = [[0 for j in range(m)] for i in range(n)]
		for j in range(0,m):
			if(S[n-1] in T[j:]):
				DP[n-1][j] = abs(m-1-j)
			else:
				DP[n-1][j] = abs(m-1-j)+1

		for i in range(0,n):
			if(T[m-1] in S[i:]):
				DP[i][m-1] = abs(n-1-i)
			else:
				DP[i][m-1] = abs(n-1-i)+1

		for i in range(n-2,-1,-1):
			for j in range(m-2,-1,-1):
				if(S[i]==T[j] or S[i] + S[i+1] == T[j]):
					DP[i][j] = DP[i+1][j+1]
				else:
					DP[i][j] = min(min(1 + DP[i+1][j], 1 + DP[i][j+1]),1 + DP[i+1][j+1])

		return DP[0][0]

	m_timestamps = 0
	a_timestamps = 0

	for x,y in itertools.zip_longest(manual,automatic):
		if(x == 't'):
			m_timestamps +=1
		if(y == 't'):
			a_timestamps +=1
	m_timestamps -=1
	a_timestamps -=1
	mpointer = 1
	apointer = 1
	edit_distance = 0
	max_WER = 0
	min_WER = 10
	avg_WER = 0
	for l in range(min(m_timestamps,a_timestamps)):
		i = mpointer
		j = apointer
		s = []
		t = []
		while True:
			if(i != len(manual) and j != len(automatic)):
				if(manual[i] == 't' and automatic[j] == 't'):
					break
				else:
					if(automatic[j] != 't'):
						s.append(automatic[j])
						j+=1
					if(manual[i] != 't'):
						t.append(manual[i])				
						i+=1
			else:
				break
		e = editD(s,t)
		edit_distance += e
		WER = (float)(e/len(t))
		if(WER < 0.3):
			avg_WER += 0.5*WER
		else:
			avg_WER += 1*WER
		if(WER > max_WER):
			max_WER = WER
		if(WER < min_WER):
			min_WER = WER
		mpointer = i+1
		apointer = j+1
	
	avg_WER = (float)(avg_WER/min(m_timestamps,a_timestamps))
	edit2Distance = editD(automatic,manual)
	WER2 = (float)(edit2Distance/len(manual))
	#print('Manual length = ',len(manual))
	#print('Automatic length = ',len(automatic))
	#print('Edit Distance = ',edit_distance)
	WER1 = (float)(edit_distance/len(manual))
	#print('Word Error Rate =',WER)
	#print('Maximum Word Error Rate =',max_WER)
	#print('Minimum Word Error Rate =',min_WER)
	WER = min(avg_WER,min(WER2,WER1))
	accuracy = (1-WER)*100
	return (WER,accuracy)



from pydub import AudioSegment
from pydub.silence import detect_nonsilent
from pydub.silence import split_on_silence

def remove_silence(input_file, output_file):
	'''
	Given an input_file path/name, this function will break into non-silence chunks
	and combine those chunks and write them to output_file path/name
	'''
	sound = AudioSegment.from_file(input_file, format = "wav")
	chunks = split_on_silence(sound, min_silence_len = 1000, silence_thresh = -24, keep_silence = 0)
	sound1 = AudioSegment.empty()
	for e in chunks:
		sound1 = sound1 + e
	sound1.export(output_file, format = "wav")
#****************************************************-----------------------------------------------------------------***********************************************************



import soundfile as sf
import numpy as np

def concat_audios(path_files, gender):
	'''
	This function will concatenate all the audios given in path_files to one mega audio(same gender)
	and then we will break this mega audio to 10s chunks and extract features
	gender can be 'm' or 'f'
	'''
	sdat = np.array([])
	samplerate = 0
	for audio in sorted(os.listdir(path_files)):
		if(audio[0] == gender):
			data, samplerate = sf.read(audio)
			sdat = np.concatenate((sdat,data),axis=0)
		
	sf.write(gender + '_fl' + '.wav', sdat, samplerate)
#******************************************************----------------------------------------------------------------**********************************************************
